/*
// This software is subject to the terms of the Eclipse Public License v1.0
// Agreement, available at the following URL:
// http://www.eclipse.org/legal/epl-v10.html.
// You must accept the terms of that agreement to use this software.
//
// Copyright (C) 2007-2014 Pentaho
// All Rights Reserved.
*/
package mondrian.olap;

import mondrian.rolap.RolapUtil;
import mondrian.test.*;

import java.io.IOException;

/**
 * <code>NullMemberRepresentationTest</code> tests the null member
 * custom representation feature supported via
 * {@link mondrian.olap.MondrianProperties#NullMemberRepresentation} property.
 * @author ajogleka
 */
public class NullMemberRepresentationTest extends FoodMartTestCase {

    public void testClosingPeriodMemberLeafWithCustomNullRepresentation() {
        assertQueryReturns(
            "with member [Measures].[Foo] as ' ClosingPeriod().uniquename '\n"
            + "select {[Measures].[Foo]} on columns,\n"
            + "  {[Time].[1997],\n"
            + "   [Time].[1997].[Q2],\n"
            + "   [Time].[1997].[Q2].[4]} on rows\n"
            + "from Sales",
            "Axis #0:\n"
            + "{}\n"
            + "Axis #1:\n"
            + "{[Measures].[Foo]}\n"
            + "Axis #2:\n"
            + "{[Time].[Time].[1997]}\n"
            + "{[Time].[Time].[1997].[Q2]}\n"
            + "{[Time].[Time].[1997].[Q2].[4]}\n"
            + "Row #0: [Time].[Time].[1997].[Q4]\n"
            + "Row #1: [Time].[Time].[1997].[Q2].[6]\n"
            + "Row #2: [Time].[Time].["
            + getNullMemberRepresentation()
            + "]\n"
            + "");
    }

    public void testItemMemberWithCustomNullMemberRepresentation()
        throws IOException
    {
        assertExprReturns(
            "[Time].[1997].Children.Item(6).UniqueName",
            "[Time].[Time].[" + getNullMemberRepresentation() + "]");
        assertExprReturns(
            "[Time].[1997].Children.Item(-1).UniqueName",
            "[Time].[Time].[" + getNullMemberRepresentation() + "]");
    }

    public void testNullMemberWithCustomRepresentation() throws IOException {
        assertExprReturns(
            "[Gender].[All Gender].Parent.UniqueName",
            "[Customer].[Gender].[" + getNullMemberRepresentation() + "]");

        assertExprReturns(
            "[Gender].[All Gender].Parent.Name", getNullMemberRepresentation());
    }

    public void testNullMemberWithCompoundOrderKey() {
        TestContext context = getTestContext().replace(
            "<Attribute name='Name' keyColumn='customer_id' nameColumn='full_name' orderByColumn='full_name' hasHierarchy='false'/>",
            "<Attribute name='Name' keyColumn='customer_id' nameColumn='full_name' orderByColumn='full_name' hasHierarchy='false'/>"
            + "<Attribute name='compoundAddress' hasHierarchy='true'>"
            + "  <Key>"
            + "    <Column name='address2'/>"
            + "  </Key>"
            + "  <Name>"
            + "    <Column name='address2'/>"
            + "  </Name>"
            + "    <OrderBy>"
            + "      <Column name='address2'/>"
            + "      <Column name='address3'/>"
            + "    </OrderBy>"
            + "</Attribute>");
        context.assertAxisReturns(
            "compoundAddress.members",
            "[Customer].[compoundAddress].[All compoundAddress]\n"
            + "[Customer].[compoundAddress].[#null]\n"
            + "[Customer].[compoundAddress].[ Apt. A]\n"
            + "[Customer].[compoundAddress].[ Unit A]\n"
            + "[Customer].[compoundAddress].[#1]\n"
            + "[Customer].[compoundAddress].[#10]\n"
            + "[Customer].[compoundAddress].[#101]\n"
            + "[Customer].[compoundAddress].[#103]\n"
            + "[Customer].[compoundAddress].[#1034]\n"
            + "[Customer].[compoundAddress].[#104]\n"
            + "[Customer].[compoundAddress].[#105]\n"
            + "[Customer].[compoundAddress].[#1067]\n"
            + "[Customer].[compoundAddress].[#108]\n"
            + "[Customer].[compoundAddress].[#11]\n"
            + "[Customer].[compoundAddress].[#110]\n"
            + "[Customer].[compoundAddress].[#1104]\n"
            + "[Customer].[compoundAddress].[#111]\n"
            + "[Customer].[compoundAddress].[#1117]\n"
            + "[Customer].[compoundAddress].[#112]\n"
            + "[Customer].[compoundAddress].[#113]\n"
            + "[Customer].[compoundAddress].[#114]\n"
            + "[Customer].[compoundAddress].[#116]\n"
            + "[Customer].[compoundAddress].[#118]\n"
            + "[Customer].[compoundAddress].[#119 A]\n"
            + "[Customer].[compoundAddress].[#12]\n"
            + "[Customer].[compoundAddress].[#120]\n"
            + "[Customer].[compoundAddress].[#1211]\n"
            + "[Customer].[compoundAddress].[#123]\n"
            + "[Customer].[compoundAddress].[#125]\n"
            + "[Customer].[compoundAddress].[#126]\n"
            + "[Customer].[compoundAddress].[#13]\n"
            + "[Customer].[compoundAddress].[#132]\n"
            + "[Customer].[compoundAddress].[#133]\n"
            + "[Customer].[compoundAddress].[#135]\n"
            + "[Customer].[compoundAddress].[#139]\n"
            + "[Customer].[compoundAddress].[#14]\n"
            + "[Customer].[compoundAddress].[#141]\n"
            + "[Customer].[compoundAddress].[#145]\n"
            + "[Customer].[compoundAddress].[#147]\n"
            + "[Customer].[compoundAddress].[#153]\n"
            + "[Customer].[compoundAddress].[#156B]\n"
            + "[Customer].[compoundAddress].[#1604]\n"
            + "[Customer].[compoundAddress].[#163]\n"
            + "[Customer].[compoundAddress].[#17]\n"
            + "[Customer].[compoundAddress].[#170]\n"
            + "[Customer].[compoundAddress].[#173]\n"
            + "[Customer].[compoundAddress].[#174]\n"
            + "[Customer].[compoundAddress].[#1741]\n"
            + "[Customer].[compoundAddress].[#178]\n"
            + "[Customer].[compoundAddress].[#18]\n"
            + "[Customer].[compoundAddress].[#180]\n"
            + "[Customer].[compoundAddress].[#19]\n"
            + "[Customer].[compoundAddress].[#2]\n"
            + "[Customer].[compoundAddress].[#20]\n"
            + "[Customer].[compoundAddress].[#201]\n"
            + "[Customer].[compoundAddress].[#2015]\n"
            + "[Customer].[compoundAddress].[#202]\n"
            + "[Customer].[compoundAddress].[#203]\n"
            + "[Customer].[compoundAddress].[#204]\n"
            + "[Customer].[compoundAddress].[#205]\n"
            + "[Customer].[compoundAddress].[#206]\n"
            + "[Customer].[compoundAddress].[#207]\n"
            + "[Customer].[compoundAddress].[#208]\n"
            + "[Customer].[compoundAddress].[#21]\n"
            + "[Customer].[compoundAddress].[#210]\n"
            + "[Customer].[compoundAddress].[#211]\n"
            + "[Customer].[compoundAddress].[#212]\n"
            + "[Customer].[compoundAddress].[#213]\n"
            + "[Customer].[compoundAddress].[#214]\n"
            + "[Customer].[compoundAddress].[#215]\n"
            + "[Customer].[compoundAddress].[#219]\n"
            + "[Customer].[compoundAddress].[#22]\n"
            + "[Customer].[compoundAddress].[#220]\n"
            + "[Customer].[compoundAddress].[#223]\n"
            + "[Customer].[compoundAddress].[#225]\n"
            + "[Customer].[compoundAddress].[#226]\n"
            + "[Customer].[compoundAddress].[#228]\n"
            + "[Customer].[compoundAddress].[#23]\n"
            + "[Customer].[compoundAddress].[#2306]\n"
            + "[Customer].[compoundAddress].[#233]\n"
            + "[Customer].[compoundAddress].[#235]\n"
            + "[Customer].[compoundAddress].[#24]\n"
            + "[Customer].[compoundAddress].[#243]\n"
            + "[Customer].[compoundAddress].[#25]\n"
            + "[Customer].[compoundAddress].[#256]\n"
            + "[Customer].[compoundAddress].[#258]\n"
            + "[Customer].[compoundAddress].[#260]\n"
            + "[Customer].[compoundAddress].[#265]\n"
            + "[Customer].[compoundAddress].[#267]\n"
            + "[Customer].[compoundAddress].[#27]\n"
            + "[Customer].[compoundAddress].[#283]\n"
            + "[Customer].[compoundAddress].[#29]\n"
            + "[Customer].[compoundAddress].[#2B]\n"
            + "[Customer].[compoundAddress].[#3]\n"
            + "[Customer].[compoundAddress].[#30]\n"
            + "[Customer].[compoundAddress].[#303]\n"
            + "[Customer].[compoundAddress].[#304]\n"
            + "[Customer].[compoundAddress].[#3055]\n"
            + "[Customer].[compoundAddress].[#307]\n"
            + "[Customer].[compoundAddress].[#308-2]\n"
            + "[Customer].[compoundAddress].[#309]\n"
            + "[Customer].[compoundAddress].[#31]\n"
            + "[Customer].[compoundAddress].[#310]\n"
            + "[Customer].[compoundAddress].[#313]\n"
            + "[Customer].[compoundAddress].[#314]\n"
            + "[Customer].[compoundAddress].[#315]\n"
            + "[Customer].[compoundAddress].[#318]\n"
            + "[Customer].[compoundAddress].[#320]\n"
            + "[Customer].[compoundAddress].[#3202]\n"
            + "[Customer].[compoundAddress].[#3213]\n"
            + "[Customer].[compoundAddress].[#326]\n"
            + "[Customer].[compoundAddress].[#331]\n"
            + "[Customer].[compoundAddress].[#333]\n"
            + "[Customer].[compoundAddress].[#335]\n"
            + "[Customer].[compoundAddress].[#338]\n"
            + "[Customer].[compoundAddress].[#339]\n"
            + "[Customer].[compoundAddress].[#34]\n"
            + "[Customer].[compoundAddress].[#340]\n"
            + "[Customer].[compoundAddress].[#35]\n"
            + "[Customer].[compoundAddress].[#36]\n"
            + "[Customer].[compoundAddress].[#360]\n"
            + "[Customer].[compoundAddress].[#38]\n"
            + "[Customer].[compoundAddress].[#4]\n"
            + "[Customer].[compoundAddress].[#40]\n"
            + "[Customer].[compoundAddress].[#404]\n"
            + "[Customer].[compoundAddress].[#409]\n"
            + "[Customer].[compoundAddress].[#42]\n"
            + "[Customer].[compoundAddress].[#422]\n"
            + "[Customer].[compoundAddress].[#425]\n"
            + "[Customer].[compoundAddress].[#439]\n"
            + "[Customer].[compoundAddress].[#44]\n"
            + "[Customer].[compoundAddress].[#45]\n"
            + "[Customer].[compoundAddress].[#46]\n"
            + "[Customer].[compoundAddress].[#49]\n"
            + "[Customer].[compoundAddress].[#495]\n"
            + "[Customer].[compoundAddress].[#4D]\n"
            + "[Customer].[compoundAddress].[#5]\n"
            + "[Customer].[compoundAddress].[#501]\n"
            + "[Customer].[compoundAddress].[#503]\n"
            + "[Customer].[compoundAddress].[#505]\n"
            + "[Customer].[compoundAddress].[#507]\n"
            + "[Customer].[compoundAddress].[#508]\n"
            + "[Customer].[compoundAddress].[#511]\n"
            + "[Customer].[compoundAddress].[#512]\n"
            + "[Customer].[compoundAddress].[#5188]\n"
            + "[Customer].[compoundAddress].[#52]\n"
            + "[Customer].[compoundAddress].[#524]\n"
            + "[Customer].[compoundAddress].[#55]\n"
            + "[Customer].[compoundAddress].[#57]\n"
            + "[Customer].[compoundAddress].[#58]\n"
            + "[Customer].[compoundAddress].[#59]\n"
            + "[Customer].[compoundAddress].[#6]\n"
            + "[Customer].[compoundAddress].[#60]\n"
            + "[Customer].[compoundAddress].[#607]\n"
            + "[Customer].[compoundAddress].[#61]\n"
            + "[Customer].[compoundAddress].[#611]\n"
            + "[Customer].[compoundAddress].[#62]\n"
            + "[Customer].[compoundAddress].[#63]\n"
            + "[Customer].[compoundAddress].[#65]\n"
            + "[Customer].[compoundAddress].[#66]\n"
            + "[Customer].[compoundAddress].[#67]\n"
            + "[Customer].[compoundAddress].[#69]\n"
            + "[Customer].[compoundAddress].[#7]\n"
            + "[Customer].[compoundAddress].[#701]\n"
            + "[Customer].[compoundAddress].[#702]\n"
            + "[Customer].[compoundAddress].[#707]\n"
            + "[Customer].[compoundAddress].[#71]\n"
            + "[Customer].[compoundAddress].[#73]\n"
            + "[Customer].[compoundAddress].[#735]\n"
            + "[Customer].[compoundAddress].[#74]\n"
            + "[Customer].[compoundAddress].[#75]\n"
            + "[Customer].[compoundAddress].[#77]\n"
            + "[Customer].[compoundAddress].[#8]\n"
            + "[Customer].[compoundAddress].[#89]\n"
            + "[Customer].[compoundAddress].[#8A]\n"
            + "[Customer].[compoundAddress].[#9]\n"
            + "[Customer].[compoundAddress].[#905]\n"
            + "[Customer].[compoundAddress].[#91]\n"
            + "[Customer].[compoundAddress].[#914]\n"
            + "[Customer].[compoundAddress].[#A]\n"
            + "[Customer].[compoundAddress].[#A211]\n"
            + "[Customer].[compoundAddress].[#B]\n"
            + "[Customer].[compoundAddress].[#C]\n"
            + "[Customer].[compoundAddress].[#D2]\n"
            + "[Customer].[compoundAddress].[#E]\n"
            + "[Customer].[compoundAddress].[#F]\n"
            + "[Customer].[compoundAddress].[#G101]\n"
            + "[Customer].[compoundAddress].[#J]\n"
            + "[Customer].[compoundAddress].[11]\n"
            + "[Customer].[compoundAddress].[117]\n"
            + "[Customer].[compoundAddress].[122]\n"
            + "[Customer].[compoundAddress].[14]\n"
            + "[Customer].[compoundAddress].[145]\n"
            + "[Customer].[compoundAddress].[2]\n"
            + "[Customer].[compoundAddress].[20]\n"
            + "[Customer].[compoundAddress].[201]\n"
            + "[Customer].[compoundAddress].[203]\n"
            + "[Customer].[compoundAddress].[21]\n"
            + "[Customer].[compoundAddress].[211]\n"
            + "[Customer].[compoundAddress].[224]\n"
            + "[Customer].[compoundAddress].[25]\n"
            + "[Customer].[compoundAddress].[324]\n"
            + "[Customer].[compoundAddress].[34]\n"
            + "[Customer].[compoundAddress].[385]\n"
            + "[Customer].[compoundAddress].[3B]\n"
            + "[Customer].[compoundAddress].[406]\n"
            + "[Customer].[compoundAddress].[6 MONTEIRA]\n"
            + "[Customer].[compoundAddress].[607]\n"
            + "[Customer].[compoundAddress].[66]\n"
            + "[Customer].[compoundAddress].[9]\n"
            + "[Customer].[compoundAddress].[A]\n"
            + "[Customer].[compoundAddress].[A211]\n"
            + "[Customer].[compoundAddress].[A4]\n"
            + "[Customer].[compoundAddress].[APT #600]\n"
            + "[Customer].[compoundAddress].[APT 1]\n"
            + "[Customer].[compoundAddress].[APT 110C]\n"
            + "[Customer].[compoundAddress].[APT 212]\n"
            + "[Customer].[compoundAddress].[APT 213]\n"
            + "[Customer].[compoundAddress].[APT 24]\n"
            + "[Customer].[compoundAddress].[APT 293]\n"
            + "[Customer].[compoundAddress].[APT 2C]\n"
            + "[Customer].[compoundAddress].[APT 308]\n"
            + "[Customer].[compoundAddress].[APT 4]\n"
            + "[Customer].[compoundAddress].[APT 98]\n"
            + "[Customer].[compoundAddress].[APT B]\n"
            + "[Customer].[compoundAddress].[APT C]\n"
            + "[Customer].[compoundAddress].[APT D]\n"
            + "[Customer].[compoundAddress].[APT F]\n"
            + "[Customer].[compoundAddress].[B105]\n"
            + "[Customer].[compoundAddress].[C]\n"
            + "[Customer].[compoundAddress].[J188]\n"
            + "[Customer].[compoundAddress].[K]\n"
            + "[Customer].[compoundAddress].[NO 138]\n"
            + "[Customer].[compoundAddress].[NO 2]\n"
            + "[Customer].[compoundAddress].[NO 4]\n"
            + "[Customer].[compoundAddress].[PSR Box #311]\n"
            + "[Customer].[compoundAddress].[Space #45]\n"
            + "[Customer].[compoundAddress].[Space 51]\n"
            + "[Customer].[compoundAddress].[Space 55]\n"
            + "[Customer].[compoundAddress].[Studio #403]\n"
            + "[Customer].[compoundAddress].[UNIT #3]\n"
            + "[Customer].[compoundAddress].[Unit 13C12]\n"
            + "[Customer].[compoundAddress].[Unit 20G]\n"
            + "[Customer].[compoundAddress].[Unit 2B]\n"
            + "[Customer].[compoundAddress].[Unit 2C]\n"
            + "[Customer].[compoundAddress].[Unit 3B]\n"
            + "[Customer].[compoundAddress].[Unit 3E]\n"
            + "[Customer].[compoundAddress].[Unit 3G]\n"
            + "[Customer].[compoundAddress].[Unit 9A]\n"
            + "[Customer].[compoundAddress].[Unit A]\n"
            + "[Customer].[compoundAddress].[Unit A309]\n"
            + "[Customer].[compoundAddress].[Unit A7]\n"
            + "[Customer].[compoundAddress].[Unit B]\n"
            + "[Customer].[compoundAddress].[Unit B-105]\n"
            + "[Customer].[compoundAddress].[Unit B308]\n"
            + "[Customer].[compoundAddress].[Unit C]\n"
            + "[Customer].[compoundAddress].[Unit C-123]\n"
            + "[Customer].[compoundAddress].[Unit C2]\n"
            + "[Customer].[compoundAddress].[Unit C220]\n"
            + "[Customer].[compoundAddress].[Unit C3]\n"
            + "[Customer].[compoundAddress].[Unit C4]\n"
            + "[Customer].[compoundAddress].[Unit C5]\n"
            + "[Customer].[compoundAddress].[Unit C7]\n"
            + "[Customer].[compoundAddress].[Unit D]\n"
            + "[Customer].[compoundAddress].[Unit D-2]\n"
            + "[Customer].[compoundAddress].[Unit D2]\n"
            + "[Customer].[compoundAddress].[Unit D4]\n"
            + "[Customer].[compoundAddress].[Unit D7]\n"
            + "[Customer].[compoundAddress].[Unit E]\n"
            + "[Customer].[compoundAddress].[Unit E1]\n"
            + "[Customer].[compoundAddress].[Unit E11]\n"
            + "[Customer].[compoundAddress].[Unit F]\n"
            + "[Customer].[compoundAddress].[Unit F13]\n"
            + "[Customer].[compoundAddress].[Unit G 202]\n"
            + "[Customer].[compoundAddress].[Unit G12]\n"
            + "[Customer].[compoundAddress].[Unit H]\n"
            + "[Customer].[compoundAddress].[Unit H103]");
    }

    private String getNullMemberRepresentation() {
        return RolapUtil.mdxNullLiteral();
    }

}

// End NullMemberRepresentationTest.java
